
/* Buffer.h - Buffer Abstraction Class (Header)
 *
 * Copyright (c) 2008 Topalis AG
 * Author: Christian Kellner <christian.kellner@topalis.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 */

#ifndef BUFFER_H
#define BUFFER_H

#include <sys/types.h>
#include <cstdlib>
#include <cstdio>
#include <cstring>

#include "Log.h"

class Buffer {

public:
    Buffer (size_t _size = 4096)
        : data (0), fill (0), size (_size)
    {
        LogThisFunc (("Allocating %u\n", size));
        data = (char *) malloc (size);
    }

    ~Buffer () {
        LogThisFunc (("Freeing\n"));
        free (data);
    }

    inline size_t GetAllocatedSize () {
        return size;
    }

    inline size_t GetSize () {
        return fill;
    }

    inline void Truncate () {
        fill = 0;
    }

    inline const void *GetData (size_t *fillOut = 0) {

        if (fillOut)
            *fillOut = fill;

        return data;
    }

    inline void Append (void *stuff, size_t len) {
        EnsureSpace (fill + len);
        memcpy (data + fill, data, len);
    }

    inline void * Reserve (size_t len) {
        char *start = data + fill;
        EnsureSpace (fill + len);
        fill += len;

        return reinterpret_cast <void *> (start);
    }

    inline void EnsureSpace (size_t freeSpace) {
        if (freeSpace > size) {
            data = (char *) realloc (data, freeSpace);
        }
    }

private:
    char *data;
    size_t   fill; /* how much data is in it */
    size_t   size /* allocated memory */;
};

#endif
