
/* HardDisk.h - HardDisk API Abstraction Class (Header)
 *
 * Copyright (c) 2008 Topalis AG
 * Author: Christian Kellner <christian.kellner@topalis.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <string>
#include <list>

#include "Buffer.h"
#include "IOError.h"
#include "VBoxFailure.h"

#ifndef HARD_DISK_H
#define HARD_DISK_H 1

class HardDisk {

public:

    virtual ~HardDisk () { } ;

    void OpenImages (std::list<std::string> &pathList);

    virtual void     OpenImage  (std::string path, unsigned int flags) = 0;
    virtual void     Read       (uint64_t offset, void *data, size_t len) = 0;
    virtual void     Write      (uint64_t offset, const void *data, size_t len) = 0;
    virtual void     Close      () = 0;
    virtual uint64_t GetSize    () = 0;
    virtual bool     IsReadOnly () = 0;
    virtual unsigned int DefaultOpenFlags () = 0;
    virtual const char *GetFormat () = 0;

    static std::list<std::string> SortFamily (std::list<std::string> &pathList);

    /* Utility Functions that operate on buffers */
    void     Write      (uint64_t offset, Buffer* const buffer);
    Buffer*  Read       (uint64_t offset, size_t len);

};

#endif

