
/* HardDiskOld.cpp - HardDisk API OLD
 *
 * Copyright (c) 2008 Topalis AG
 * Author: Christian Kellner <christian.kellner@topalis.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <iprt/runtime.h>
#include <iprt/stream.h>
#include <iprt/string.h>
#include <iprt/asm.h>
#include <iprt/uuid.h>
#include <iprt/thread.h>
#include <iprt/path.h>
#include <iprt/param.h>
#include <iprt/dir.h>
#include <iprt/file.h>
#include <iprt/env.h>
#include <iprt/cidr.h>
#include <VBox/err.h>
#include <VBox/version.h>

#include <cstdio>

#include "IOError.h"
#include "HardDiskOld.h"

#define printf RTPrintf

HardDiskOld::HardDiskOld ()
    : diskHandle (NULL), nImages (0)
{

}

HardDiskOld::~HardDiskOld ()
{
    if (diskHandle) {
        Close ();
    }
}

void
HardDiskOld::Initilize ()
{
    diskHandle = VDIDiskCreate();
}

void
HardDiskOld::OpenImage (std::string path, unsigned int flags)
{
    if (diskHandle == NULL) {
       Initilize ();
    }

    int vrc = VDIDiskOpenImage (diskHandle, path.c_str (), flags);

    if (VBOX_FAILURE (vrc)) {
        throw VBoxFailure (vrc);
    }

    nImages++;
}

void
HardDiskOld::Read (uint64_t offset, void *data, size_t len)
{
    int vrc = VDIDiskRead (diskHandle, offset, data, len);

    if (VBOX_FAILURE (vrc))
        throw VBoxFailure (vrc);
}


void
HardDiskOld::Write (uint64_t offset, const void *data, size_t len)
{
    int vrc = VDIDiskWrite (diskHandle, offset, data, len);

    if (VBOX_FAILURE (vrc))
        throw VBoxFailure (vrc);
}

uint64_t
HardDiskOld::GetSize ()
{
    return VDIDiskGetSize (diskHandle);
}

void
HardDiskOld::Close ()
{
    if (diskHandle) {
       VDIDiskCloseImage (diskHandle);
       diskHandle = NULL;
    }
}

bool
HardDiskOld::IsReadOnly ()
{
    return VDIDiskIsReadOnly (diskHandle);
}

const char *
HardDiskOld::GetFormat ()
{
    return "VDI";
}
