
/* IOError.cpp - IOError Exception
 *
 * Copyright (c) 2008 Topalis AG
 * Author: Christian Kellner <christian.kellner@topalis.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <cerrno>
#include <cstring>
#include <cstdio>

#include "IOError.h"

IOError::IOError (int num)
    : errorNumber (num)
{
    strerror_r (errorNumber, errorString, sizeof (errorString));
}

IOError::IOError (const char *desc, int num)
    : errorNumber (num)
{
    char errorDesc[1024] = {0, };

    strerror_r (num, errorDesc, sizeof (errorDesc));
    snprintf (errorString, sizeof (errorString), "IOError: %s (%d) %s",
              desc, num, errorDesc);
}

const char *
IOError::what () const throw ()
{
    return errorString;
}
