
/* Nbd.h - Network Block Device Abstraction Class (Header)
 *
 * Copyright (c) 2008 Topalis AG
 * Author: Christian Kellner <christian.kellner@topalis.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <string>
#include <iprt/thread.h>

#include <fcntl.h>
#include <sys/types.h>

#include "IOError.h"
#include "Socket.h"

#ifndef NBD_H
#define NBD_H


class Nbd {

    /*  */
public:
    class Reply;

    class Request {
    public:
        enum RequestType {
            CmdDisconnect = 0,
            CmdRead,
            CmdWrite
        };

        RequestType type;
        uint64_t    offset;
        uint64_t    lenght;

        static Request RecieveNext (Socket &sock);
        Reply *        CreateReply ();
        Buffer *       GetPayload ();


    private:
        Request (Socket &sock, char handle[8]);
        Socket &sock;
        char handle[8];
    };

    class Reply {

    public:

        ~Reply () { if (payload) delete payload; }
               void Send       (void);
               void SetPayload (Buffer *data);
        inline void SetError   (int err) {error = err;}

        int     error;

    private:
        char handle[8];
        Socket &sock;
        Reply (Socket &sock, char handle[8]);
        Buffer *payload;
        friend class Request;
    };

    /* ** */

public:
    Nbd (const char *dev)
        : fd (-1), devicePath (dev) { }
    ~Nbd ();

    void Open (int flags = O_RDWR);
    void ClearQueue (bool ignoreErrors = false);
    void Disconnect ();
    void ClearSocket (bool ignoreErrors = false);
    void SetBlockSize (unsigned long blocksize = 512);
    void SetSizeBlocks (unsigned long blocks);
    void SetReadOnly (bool ro);
    void SetTimeOut (unsigned long timeout);
    void SetSocket (int socket);
    int DoIt ();

    /* Async (threaded versions) */
    void DoItBegin ();
    int  DoItEnd ();

    int Poke ();
    void PokeSync (int timeout);
    int Close ();

private:
    static int PokeThread (RTTHREAD ThreadSelf, void *userData);

private:
    int fd;
    int sock;
    std::string devicePath;
    RTTHREAD doItThread;
    RTTHREAD pokeThread;

};

#endif

